@@ -20,12 +20,13 @@ __all__ = ["Attention", "string_to_softmax", "QuantizedAttention"]
 import keras
 import tensorflow as tf
 
-from .decorators import register_quantize_target, register_aligned_inputs
+from .layers_base import (register_quantize_target, register_aligned_inputs, apply_buffer_bitwidth,
+                          QuantizedLayer)
 from .reshaping import QuantizedReshape, QuantizedPermute
 from .shiftmax import shiftmax, QuantizedShiftmax
 from .recorders import TensorRecorder
 from .quantizers import OutputQuantizer
-from ..tensors import FixedPoint, MAX_BUFFER_BITWIDTH, round_log2, pow2
+from ..tensors import FixedPoint, round_log2, pow2
 
 
 def string_to_softmax(s):
@@ -100,6 +101,7 @@ class Attention(keras.layers.Layer):
         self.softmax_op = string_to_softmax(self.softmax)
 
     def build(self, input_shape):
+        super().build(input_shape)
         assert len(input_shape) == 3
         self.hidden_size = input_shape[0][-1]
         if self.hidden_size % self.num_heads != 0:
@@ -154,7 +156,7 @@ class Attention(keras.layers.Layer):
 @register_quantize_target(Attention)
 @register_aligned_inputs
 @tf.keras.utils.register_keras_serializable()
-class QuantizedAttention(Attention):
+class QuantizedAttention(QuantizedLayer, Attention):
     """An Attention layer that operates on quantized inputs
 
     Args:
@@ -162,16 +164,12 @@ class QuantizedAttention(Attention):
         quant_config (dict, optional): the serialized quantization configuration. Defaults to None.
         softmax (str, optional): 'softmax' or 'shiftmax'. Defaults to 'shiftmax'
     """
+    arg_constraints = {'softmax': 'shiftmax'}
 
     def __init__(self, num_heads, quant_config=None, softmax='shiftmax', **kwargs):
-        if softmax != 'shiftmax':
-            raise ValueError(
-                "Only shiftmax is supported for quantized attention")
-        super().__init__(num_heads=num_heads, softmax=softmax, **kwargs)
-        self.quant_config = quant_config
-        quant_config = quant_config or dict()
+        super().__init__(num_heads=num_heads, softmax=softmax, quant_config=quant_config, **kwargs)
 
-        out_quant_cfg = quant_config.get("output_quantizer", False)
+        out_quant_cfg = self.quant_config.get("output_quantizer", False)
         if out_quant_cfg:
             self.out_quantizer = OutputQuantizer(
                 name="output_quantizer", **out_quant_cfg)
@@ -179,12 +177,12 @@ class QuantizedAttention(Attention):
             self.out_quantizer = None
 
         # Override softmax operation
-        softmax_quant_conf = quant_config.get("softmax", None)
+        softmax_quant_conf = self.quant_config.get("softmax", None)
         self.softmax_op = QuantizedShiftmax(
             quant_config=softmax_quant_conf, name="quantized_softmax")
-        self.buffer_bitwidth = MAX_BUFFER_BITWIDTH - 1
+        self.buffer_bitwidth = apply_buffer_bitwidth(self.quant_config, signed=True)
         # Add objects that will store the shift values.
-        self.values_shift = TensorRecorder()
+        self.values_shift = TensorRecorder(self.name + "/value_shift")
 
     def separate_heads(self, x):
         x = QuantizedReshape((-1, self.num_heads, self.dim))(x)
@@ -220,8 +218,3 @@ class QuantizedAttention(Attention):
         if self.out_quantizer is not None:
             output = self.out_quantizer(output)
         return output, mask
-
-    def get_config(self):
-        config = super().get_config()
-        config["quant_config"] = self.quant_config
-        return config
