@@ -18,27 +18,27 @@
 __all__ = ["PaddedConv2D", "QuantizedConv2D", "QuantizedConv2DTranspose"]
 
 import tensorflow as tf
+import numpy as np
 
 from keras import layers, backend
-from keras.utils import conv_utils
 
-from .decorators import (register_quantize_target, rescale_outputs, tensor_inputs,
-                         neural_layer_init, register_aligned_inputs)
+from .layers_base import (register_quantize_target, rescale_outputs, tensor_inputs,
+                          neural_layer_init, register_aligned_inputs, QuantizedLayer)
 from .quantizers import AlignedWeightQuantizer
 from ..tensors import QTensor, QFloat, FixedPoint
 
 
-def apply_padding(inputs, strides, kernel_size, padding_value):
+def apply_padding(inputs, strides, kernel_size, padding_values):
     """Apply "SAME" padding to the inputs
 
     Args:
-        inputs (:obj:`QFloat`): the inputs tensor.
+        inputs (:obj:`QFloat`, tf.Tensor): the inputs tensor.
         strides (tuple): the strides tuple.
         kernel_size (int): the kernel size.
-        padding_value (:obj:`QFloat`): the padding value to apply.
+        padding_values (:obj:`QFloat`, tf.Tensor): the padding values to apply.
 
     Returns:
-        tf.Tensor: padded inputs.
+        tf.Tensor or :obj:`QFloat`: padded inputs.
     """
     _, h, w, _ = inputs.shape
     filter_width = kernel_size[0]
@@ -57,42 +57,104 @@ def apply_padding(inputs, strides, kernel_size, padding_value):
     pad_right = pad_along_width - pad_left
     padding = [[0, 0], [pad_top, pad_bottom], [pad_left, pad_right], [0, 0]]
 
-    return tf.pad(inputs, padding, "CONSTANT", padding_value)
+    if padding_values.shape.ndims == 0:
+        # per-tensor algorithm is faster than per-axis one
+        output = tf.pad(inputs, padding, "CONSTANT", padding_values)
+    else:
+        # The per-channel algorithm consists of:
+        # - subtract the padding value to inputs
+        # - add a zero padding
+        # - add the padding value to revert the subtraction from inputs and add the desired value
+        #   around
+        inputs = inputs - padding_values
+        zero_pad = 0.
+        if isinstance(inputs, QFloat):
+            zero_pad = QFloat(FixedPoint(0, inputs.value_bits, inputs.frac_bits), inputs.scales)
+        output = tf.pad(inputs, padding, "CONSTANT", zero_pad)
+        output = output + padding_values
+
+    return output
+
+
+def deconv_output_length(input_length, filter_size, padding,
+                         output_padding=None, stride=0, dilation=1):
+    """Determines output length of a transposed convolution given input length.
+
+    Args:
+        input_length (int): length of input.
+        filter_size (int): size of the kernel.
+        padding (str): one of `"same"`, `"valid"`, `"full"`.
+        output_padding (int, optional): amount of padding along the output dimension. Can be set to
+            `None` in which case the output length is inferred. Defaults to None.
+        stride (int, optional): stride value. Defaults to 0.
+        dilation (int, optional): dilation value. Defaults to 1.
+
+    Returns:
+        int: the output length
+    """
+    assert padding in {"same", "valid", "full"}
+    if input_length is None:
+        return None
+
+    # Get the dilated kernel size
+    filter_size = filter_size + (filter_size - 1) * (dilation - 1)
+
+    # Infer length if output padding is None, else compute the exact length
+    if output_padding is None:
+        if padding == "valid":
+            length = input_length * stride + max(filter_size - stride, 0)
+        elif padding == "full":
+            length = input_length * stride - (stride + filter_size - 2)
+        elif padding == "same":
+            length = input_length * stride
+
+    else:
+        if padding == "same":
+            pad = filter_size // 2
+        elif padding == "valid":
+            pad = 0
+        elif padding == "full":
+            pad = filter_size - 1
+
+        length = ((input_length - 1) * stride + filter_size - 2 * pad + output_padding)
+    return length
 
 
 @tf.keras.utils.register_keras_serializable()
 class PaddedConv2D(layers.Conv2D):
-    """A convolutional layer that can use a custom padding value.
+    """A convolutional layer that can use custom padding values.
 
-    Note that when a padding value is provided, padding 'SAME' will be applied with the provided
+    Note that when padding values are provided, padding 'SAME' will be applied with the provided
     value (overriding 'padding' parameter).
 
     Args:
-        padding_value (float, optional): the value used when padding for the 'same' convolution
-            type. If None, zero-padding is used. Defaults to None.
+        padding_value (float, list, tensor, optional): the value or the list of values used when
+            padding for the 'same' convolution type. Padding is per-tensor if one value is provided
+            or per-channel otherwise. If None, zero-padding is used. Defaults to None.
     """
 
     def __init__(self, *args, padding_value=None, **kwargs):
         super().__init__(*args, **kwargs)
-        # Raise an error if the padding value is not a scalar
+        self._padding_value = padding_value
         if padding_value is not None:
-            if tf.size(padding_value) != 1:
-                raise ValueError(f"The padding value must be a scalar. Receives {padding_value}.")
             # When a custom padding_value is given, self.padding is overwritten and 'SAME' padding
             # will be done explicitly in the call.
-            self._padding_value = tf.constant(padding_value)
             self.padding = 'valid'
-        else:
-            self._padding_value = None
 
     def call(self, inputs):
         # We need a custom padding for specifics padding values
         if self._padding_value is not None:
-            # Note that we use the custom padding even when padding value is 0.
+            # Raise an error if the padding values have not the expected number of values
+            pad_size = np.size(self._padding_value)
+            if pad_size not in (1, inputs.shape[-1]):
+                raise ValueError("The padding value must be a scalar or a list of values, for "
+                                 f"a value per input channel. Receives {self._padding_value}.")
+            t_padding = tf.cast(self._padding_value, dtype=tf.float32)
+            # Note that we use the custom padding even when padding value provided is 0.
             inputs = apply_padding(inputs,
                                    list(self.strides),
                                    (self.kernel.shape[1], self.kernel.shape[0]),
-                                   self._padding_value)
+                                   t_padding)
 
         outputs = self.convolution_op(inputs, self.kernel)
 
@@ -109,26 +171,26 @@ class PaddedConv2D(layers.Conv2D):
 @register_quantize_target([layers.Conv2D, PaddedConv2D])
 @register_aligned_inputs
 @tf.keras.utils.register_keras_serializable()
-class QuantizedConv2D(layers.Conv2D):
+class QuantizedConv2D(QuantizedLayer, layers.Conv2D):
     """A convolutional layer that operates on quantized inputs and weights.
 
-    Note that when a padding value is provided, padding 'SAME' will be applied with the provided
+    Note that when padding values are provided, padding 'SAME' will be applied with the provided
     value (overriding 'padding' parameter).
 
     Args:
         quant_config (dict, optional): the serialized quantization configuration. Defaults to None.
-        padding_value (float, optional): the value used when padding for the 'same' convolution
-            type. If None, zero-padding is used. Defaults to None.
+        padding_value (float, list, tensor, optional): the value or the list of values used when
+            padding for the 'same' convolution type. Padding is per-tensor if one value is provided
+            or per-channel otherwise. If None, zero-padding is used. Defaults to None.
     """
     @neural_layer_init(False)
     def __init__(self, *args, padding_value=None, quant_config=None, **kwargs):
-        if padding_value is None:
-            self.padding_value = None
-        else:
-            self.padding_value = float(padding_value)
-            # Raise an error if the padding value is not a scalar
-            if tf.size(padding_value) != 1:
-                raise ValueError(f"The padding value must be a scalar. Receives {padding_value}.")
+        # Limit buffer bitwidth to 27
+        self.quant_config['buffer_bitwidth'] = min(28, self.quant_config['buffer_bitwidth'])
+        self.buffer_bitwidth = self.quant_config['buffer_bitwidth'] - 1
+
+        self.padding_value = padding_value
+        if padding_value is not None:
             # We quantize the padding_value with the same bitwidth as the weights
             self.padding_value_quantizer = AlignedWeightQuantizer(self.weight_quantizer.bitwidth,
                                                                   signed=True)
@@ -149,14 +211,20 @@ class QuantizedConv2D(layers.Conv2D):
             # wrap the inputs in a QFloat of scale 1.
             if not isinstance(inputs, QFloat):
                 raise ValueError("When using a non-zero padding value, the inputs must be QFloat.")
-            # Quantize and align padding_value on the inputs
-            padding_value = self.padding_value_quantizer(self.padding_value, inputs)
 
-            # Note that we use the custom padding even when padding value is 0.
+            pad_size = np.size(self.padding_value)
+            if pad_size not in (1, inputs.shape[-1]):
+                raise ValueError("The padding value must be a scalar or a list of values, for "
+                                 f"a value per input channel. Receives {self.padding_value}.")
+            t_padding_value = tf.cast(self.padding_value, dtype=tf.float32)
+            # Quantize and align padding_value on the inputs
+            q_padding_value = self.padding_value_quantizer(t_padding_value, inputs)
+
+            # Note that we use the custom padding even when padding value provided is 0.
             inputs = apply_padding(inputs,
                                    list(self.strides),
                                    (kernel.shape[1], kernel.shape[0]),
-                                   padding_value)
+                                   q_padding_value)
 
         if isinstance(self.padding, str):
             tf_padding = self.padding.upper()
@@ -180,7 +248,6 @@ class QuantizedConv2D(layers.Conv2D):
 
     def get_config(self):
         config = super().get_config()
-        config["quant_config"] = self.quant_config
         if self.padding_value is not None:
             config["padding_value"] = self.padding_value
         return config
@@ -189,7 +256,7 @@ class QuantizedConv2D(layers.Conv2D):
 @register_quantize_target(layers.Conv2DTranspose)
 @register_aligned_inputs
 @tf.keras.utils.register_keras_serializable()
-class QuantizedConv2DTranspose(layers.Conv2DTranspose):
+class QuantizedConv2DTranspose(QuantizedLayer, layers.Conv2DTranspose):
     """A transposed convolutional layer that operates on quantized inputs and weights.
 
     Args:
@@ -197,7 +264,9 @@ class QuantizedConv2DTranspose(layers.Conv2DTranspose):
     """
     @neural_layer_init(False)
     def __init__(self, *args, quant_config=None, **kwargs):
-        pass
+        # Limit buffer bitwidth to 27
+        self.quant_config['buffer_bitwidth'] = min(28, self.quant_config['buffer_bitwidth'])
+        self.buffer_bitwidth = self.quant_config['buffer_bitwidth'] - 1
 
     @tensor_inputs([FixedPoint])
     @rescale_outputs
@@ -222,18 +291,18 @@ class QuantizedConv2DTranspose(layers.Conv2DTranspose):
             out_pad_h, out_pad_w = self.output_padding
 
         # Infer the dynamic output shape
-        out_height = conv_utils.deconv_output_length(height,
-                                                     kernel_h,
-                                                     padding=self.padding,
-                                                     output_padding=out_pad_h,
-                                                     stride=stride_h,
-                                                     dilation=self.dilation_rate[0])
-        out_width = conv_utils.deconv_output_length(width,
-                                                    kernel_w,
-                                                    padding=self.padding,
-                                                    output_padding=out_pad_w,
-                                                    stride=stride_w,
-                                                    dilation=self.dilation_rate[1])
+        out_height = deconv_output_length(height,
+                                          kernel_h,
+                                          padding=self.padding,
+                                          output_padding=out_pad_h,
+                                          stride=stride_h,
+                                          dilation=self.dilation_rate[0])
+        out_width = deconv_output_length(width,
+                                         kernel_w,
+                                         padding=self.padding,
+                                         output_padding=out_pad_w,
+                                         stride=stride_w,
+                                         dilation=self.dilation_rate[1])
 
         output_shape = (batch_size, out_height, out_width, self.filters)
         output_shape_tensor = tf.stack(output_shape)
@@ -250,8 +319,3 @@ class QuantizedConv2DTranspose(layers.Conv2DTranspose):
             outputs = tf.add(outputs, bias)
 
         return outputs
-
-    def get_config(self):
-        config = super().get_config()
-        config["quant_config"] = self.quant_config
-        return config
