@@ -20,15 +20,15 @@ __all__ = ["QuantizedDense"]
 import tensorflow as tf
 import keras
 
-from .decorators import (register_quantize_target, rescale_outputs, tensor_inputs,
-                         neural_layer_init, register_aligned_inputs)
+from .layers_base import (register_quantize_target, rescale_outputs, tensor_inputs,
+                          neural_layer_init, register_aligned_inputs, QuantizedLayer)
 from ..tensors import QTensor
 
 
 @register_quantize_target(keras.layers.Dense)
 @register_aligned_inputs
 @tf.keras.utils.register_keras_serializable()
-class QuantizedDense(keras.layers.Dense):
+class QuantizedDense(QuantizedLayer, keras.layers.Dense):
     """A Dense layer that operates on quantized inputs and weights
 
     Args:
@@ -36,7 +36,9 @@ class QuantizedDense(keras.layers.Dense):
     """
     @neural_layer_init(False)
     def __init__(self, *args, quant_config=None, **kwargs):
-        pass
+        # Limit buffer bitwidth to 27 for HW constraint
+        self.quant_config['buffer_bitwidth'] = min(28, self.quant_config['buffer_bitwidth'])
+        self.buffer_bitwidth = self.quant_config['buffer_bitwidth'] - 1
 
     @tensor_inputs([QTensor, tf.Tensor])
     @rescale_outputs
@@ -52,8 +54,3 @@ class QuantizedDense(keras.layers.Dense):
             outputs = tf.add(outputs, bias)
 
         return outputs
-
-    def get_config(self):
-        config = super().get_config()
-        config["quant_config"] = self.quant_config
-        return config
