@@ -20,14 +20,15 @@ __all__ = ["QuantizedDropout"]
 import keras
 import tensorflow as tf
 
-from .decorators import register_quantize_target, tensor_inputs, register_no_output_quantizer
+from .layers_base import (register_quantize_target, tensor_inputs, register_no_output_quantizer,
+                          QuantizedLayer)
 from ..tensors import QTensor
 
 
 @register_quantize_target(keras.layers.Dropout)
 @register_no_output_quantizer
 @tf.keras.utils.register_keras_serializable()
-class QuantizedDropout(keras.layers.Dropout):
+class QuantizedDropout(QuantizedLayer, keras.layers.Dropout):
     """ A dropout layer that operates on quantized inputs and weights.
 
     It is only implemented as a passthrough.
@@ -37,3 +38,6 @@ class QuantizedDropout(keras.layers.Dropout):
     def call(self, inputs):
         # QuantizedDropout act as a pass through.
         return inputs
+
+    def get_config(self):
+        return keras.layers.Dropout.get_config(self)
