@@ -21,11 +21,11 @@ import tensorflow as tf
 import keras
 from tensorflow.python.framework import ops
 
-from .decorators import (register_quantize_target, rescale_outputs, tensor_inputs,
-                         register_aligned_inputs)
+from .layers_base import (register_quantize_target, rescale_outputs, tensor_inputs,
+                          register_aligned_inputs, apply_buffer_bitwidth, QuantizedLayer)
 from .recorders import TensorRecorder
 from .quantizers import AlignedWeightQuantizer, WeightQuantizer, OutputQuantizer
-from ..tensors import FixedPoint, BASE_BUFFER_BITWIDTH
+from ..tensors import FixedPoint
 
 
 @tf.keras.utils.register_keras_serializable()
@@ -118,31 +118,31 @@ class LayerMadNormalization(keras.layers.LayerNormalization):
 @register_quantize_target(LayerMadNormalization)
 @register_aligned_inputs
 @tf.keras.utils.register_keras_serializable()
-class QuantizedLayerNormalization(LayerMadNormalization):
+class QuantizedLayerNormalization(QuantizedLayer, LayerMadNormalization):
     """ A LayerNormalization layer that operates on quantized inputs and weights.
 
     Args:
         quant_config (dict, optional): the serialized quantization configuration. Defaults to None.
     """
+
     def __init__(self, *args, quant_config=None, **kwargs):
-        super().__init__(*args, **kwargs)
-        quant_config = quant_config or dict()
-        self.quant_config = quant_config
-        out_quant_cfg = quant_config.get("output_quantizer", False)
+        super().__init__(*args, quant_config=quant_config, **kwargs)
+        out_quant_cfg = self.quant_config.get("output_quantizer", False)
         if out_quant_cfg:
             self.out_quantizer = OutputQuantizer(
                 name="output_quantizer", **out_quant_cfg)
         else:
             self.out_quantizer = None
-        self.buffer_bitwidth = BASE_BUFFER_BITWIDTH - 1
-        gamma_quantizer_cfg = quant_config.get("gamma_quantizer", {"bitwidth": 8})
-        self.quant_config["gamma_quantizer"] = gamma_quantizer_cfg
+        self.buffer_bitwidth = apply_buffer_bitwidth(self.quant_config, signed=True)
+        if "gamma_quantizer" not in self.quant_config:
+            self.quant_config["gamma_quantizer"] = {"bitwidth": 8}
+        gamma_quantizer_cfg = self.quant_config["gamma_quantizer"]
         self.gamma_quantizer = WeightQuantizer(name="gamma_quantizer", **gamma_quantizer_cfg)
-        beta_quantizer_cfg = quant_config.get("beta_quantizer", {})
+        beta_quantizer_cfg = self.quant_config.get("beta_quantizer", {})
         self.beta_quantizer = AlignedWeightQuantizer(name="beta_quantizer", **beta_quantizer_cfg)
 
         # Add objects that will store the shift values.
-        self.gamma_shift = TensorRecorder()
+        self.gamma_shift = TensorRecorder(self.name + "/gamma_shift")
 
     @tensor_inputs([FixedPoint])
     @rescale_outputs
@@ -202,8 +202,3 @@ class QuantizedLayerNormalization(LayerMadNormalization):
 
         # Evaluate outputs
         return tf.add(inputs_rescaled, beta)
-
-    def get_config(self):
-        config = super().get_config()
-        config["quant_config"] = self.quant_config
-        return config
