@@ -20,18 +20,17 @@ __all__ = ["QuantizedMaxPool2D", "QuantizedGlobalAveragePooling2D"]
 import tensorflow as tf
 
 from keras.layers import MaxPool2D, GlobalAveragePooling2D
-from keras.utils import conv_utils
 
-from .decorators import (register_quantize_target, register_no_output_quantizer, rescale_outputs,
-                         tensor_inputs)
+from .layers_base import (register_quantize_target, register_no_output_quantizer, rescale_outputs,
+                          tensor_inputs, apply_buffer_bitwidth, QuantizedLayer)
 from .quantizers import OutputQuantizer
-from ..tensors import FixedPoint, QTensor, QFloat, MAX_BUFFER_BITWIDTH
+from ..tensors import FixedPoint, QTensor, QFloat
 
 
 @register_quantize_target(MaxPool2D)
 @register_no_output_quantizer
 @tf.keras.utils.register_keras_serializable()
-class QuantizedMaxPool2D(MaxPool2D):
+class QuantizedMaxPool2D(QuantizedLayer, MaxPool2D):
     """A max pooling layer that operates on quantized inputs.
 
     """
@@ -40,20 +39,24 @@ class QuantizedMaxPool2D(MaxPool2D):
         if self.data_format == "channels_last":
             ksize = (1,) + self.pool_size + (1,)
             strides = (1,) + self.strides + (1,)
+            data_format = "NHWC"
         else:
             ksize = (1, 1) + self.pool_size
             strides = (1, 1) + self.strides
+            data_format = "NCHW"
 
-        data_format = conv_utils.convert_data_format(self.data_format, 4)
         padding = self.padding.upper()
         outputs = tf.nn.max_pool(inputs, ksize=ksize, strides=strides, padding=padding,
                                  data_format=data_format)
         return outputs
 
+    def get_config(self):
+        return MaxPool2D.get_config(self)
+
 
 @register_quantize_target(GlobalAveragePooling2D)
 @tf.keras.utils.register_keras_serializable()
-class QuantizedGlobalAveragePooling2D(GlobalAveragePooling2D):
+class QuantizedGlobalAveragePooling2D(QuantizedLayer, GlobalAveragePooling2D):
     """A global average pooling layer that operates on quantized inputs.
 
      Args:
@@ -61,16 +64,14 @@ class QuantizedGlobalAveragePooling2D(GlobalAveragePooling2D):
     """
 
     def __init__(self, quant_config=None, **kwargs):
-        super().__init__(**kwargs)
-        self.quant_config = quant_config
-        quant_config = quant_config or dict()
-        out_quant_cfg = quant_config.get("output_quantizer", False)
+        super().__init__(quant_config=quant_config, **kwargs)
+        out_quant_cfg = self.quant_config.get("output_quantizer", False)
         if out_quant_cfg:
             self.out_quantizer = OutputQuantizer(
                 name="output_quantizer", **out_quant_cfg)
         else:
             self.out_quantizer = None
-        self.buffer_bitwidth = MAX_BUFFER_BITWIDTH
+        self.buffer_bitwidth = apply_buffer_bitwidth(self.quant_config, signed=False)
 
     def build(self, input_shape):
         super().build(input_shape)
@@ -89,8 +90,3 @@ class QuantizedGlobalAveragePooling2D(GlobalAveragePooling2D):
         if isinstance(inputs, FixedPoint):
             return QFloat(inputs_sum, self.spatial_size_rec)
         return inputs_sum / self.spatial_size
-
-    def get_config(self):
-        config = super().get_config()
-        config.update({"quant_config": self.quant_config})
-        return config
