@@ -40,8 +40,8 @@ class AlignedWeightQuantizer(Quantizer):
     def __init__(self, bitwidth=8, signed=True, **kwargs):
         super().__init__(bitwidth, signed, **kwargs)
         # Add the object that will store the shift values.
-        self.shift = TensorRecorder()
-        self.qweights = QFloatRecorder()
+        self.shift = TensorRecorder(self.name + "/shift")
+        self.qweights = QFloatRecorder(self.name + "/qweights")
 
     def call(self, inputs, other):
         """Quantize the float inputs, aligned on another QFloat
@@ -71,8 +71,22 @@ class AlignedWeightQuantizer(Quantizer):
 
         other_value_bits = other.fp.value_bits
         other_frac_bits = other.fp.frac_bits
-        # Compute the symmetric quantization ranges from the inputs
-        ranges = tf.math.reduce_max(tf.math.abs(inputs))
+
+        # Compute the symmetric quantization ranges from the inputs. When the bitwidth
+        # is greater than 4 (e.g. 8 bits), experimentations have shown that models quantized
+        # per axis (especially for the bias) have a better performance.
+        # However for the 4bits, the performance was lower if we quantize per axis that's why
+        # we keep it per tensor.
+        if self.bitwidth > 4:
+            # if the inputs are 0D or 1D vector, the ranges will have the same shape as the inputs.
+            # In the other case for 2D or 3D (e.g. PositionEmbs), we reduce all axis
+            # except the last.
+            ndims = len(tf.shape(inputs))
+            axis = list(range(ndims - 1))
+        else:
+            axis = None
+
+        ranges = tf.math.reduce_max(tf.math.abs(inputs), axis=axis)
         # Evaluate the maximum fractional bits we can use for the specified ranges
         frac_bits = QFloat.max_frac_bits(self.value_bits, ranges, other.scales)
         # Remove the input-dependent fractional bits from the gradient tape to avoid a loopback
