@@ -17,17 +17,19 @@
 
 __all__ = ["QuantizedRescaling"]
 
+import keras.layers
 import tensorflow as tf
 import keras
 
-from .decorators import register_quantize_target, register_no_output_quantizer
-from ..tensors import FixedPoint, QFloat
+from .layers_base import (register_quantize_target, register_no_output_quantizer, tensor_inputs,
+                          QuantizedLayer)
+from ..tensors import QFloat
 
 
 @register_quantize_target(keras.layers.Rescaling)
 @register_no_output_quantizer
 @tf.keras.utils.register_keras_serializable()
-class QuantizedRescaling(keras.layers.Rescaling):
+class QuantizedRescaling(QuantizedLayer, keras.layers.Rescaling):
     """A layer that multiplies integer inputs by a scale
 
     This is a simplified version of the keras Rescaling layer:
@@ -49,8 +51,10 @@ class QuantizedRescaling(keras.layers.Rescaling):
         if tf.reduce_any(self.offset != 0):
             raise ValueError("QuantizedRescaling only accepts zero offset.")
 
+    @tensor_inputs([tf.Tensor])
     def call(self, inputs):
-        # Assume the inputs are 8-bit unsigned integer
-        fp = FixedPoint(tf.round(inputs), 8, 0)
         # Wrap them into a QFloat with the specified scale
-        return QFloat(fp, self.scale)
+        return QFloat(inputs, self.scale)
+
+    def get_config(self):
+        return keras.layers.Rescaling.get_config(self)
