@@ -20,7 +20,7 @@ __all__ = ["QuantizedReshape"]
 import tensorflow as tf
 import keras
 
-from ..decorators import register_quantize_target, tensor_inputs, register_no_output_quantizer
+from ..layers_base import register_quantize_target, tensor_inputs, register_no_output_quantizer
 from ...tensors import QTensor, FixedPoint
 
 
