@@ -22,33 +22,27 @@ __all__ = ["QuantizedSeparableConv2D"]
 
 import tensorflow as tf
 from keras.layers import SeparableConv2D
-from keras.utils import conv_utils
 from keras import backend
 
-from .decorators import (register_quantize_target, rescale_outputs, tensor_inputs,
-                         neural_layer_init, register_aligned_inputs)
+from .layers_base import (register_quantize_target, rescale_outputs, tensor_inputs,
+                          neural_layer_init, register_aligned_inputs, QuantizedLayer)
 from ..tensors import FixedPoint
 
 
 @register_quantize_target(SeparableConv2D)
 @register_aligned_inputs
 @tf.keras.utils.register_keras_serializable()
-class QuantizedSeparableConv2D(SeparableConv2D):
+class QuantizedSeparableConv2D(QuantizedLayer, SeparableConv2D):
     """ A separable convolutional layer that operates on quantized inputs and weights.
 
     Args:
         quant_config (dict, optional): the serialized quantization configuration. Defaults to None.
     """
+    arg_constraints = {'depth_multiplier': 1}
+
     @neural_layer_init(True)
     def __init__(self, *args, quant_config=None, **kwargs):
-        if 'dilation_rate' in kwargs:
-            if kwargs['dilation_rate'] not in [1, [1, 1], (1, 1)]:
-                raise ValueError("Keyword argument 'dilation_rate' is not supported in \
-                                 QuantizedSeparableConv2D.")
-        if 'depth_multiplier' in kwargs:
-            if kwargs['depth_multiplier'] != 1:
-                raise ValueError("Keyword argument 'depth_multiplier' is not supported in \
-                                 QuantizedSeparableConv2D.")
+        pass
 
     @tensor_inputs([FixedPoint, tf.Tensor])
     @rescale_outputs
@@ -74,7 +68,7 @@ class QuantizedSeparableConv2D(SeparableConv2D):
             pointwise_kernel,
             strides=[1, 1, 1, 1],
             padding='VALID',
-            data_format=conv_utils.convert_data_format(self.data_format, ndim=4))
+            data_format="NHWC" if self.data_format == "channels_last" else "NCHW")
 
         if self.use_bias:
             # Quantize bias and align it on the outputs
@@ -82,8 +76,3 @@ class QuantizedSeparableConv2D(SeparableConv2D):
             outputs = tf.add(outputs, bias)
 
         return outputs
-
-    def get_config(self):
-        config = super().get_config()
-        config["quant_config"] = self.quant_config
-        return config
