@@ -20,10 +20,10 @@ __all__ = ["shiftmax", "Shiftmax", "QuantizedShiftmax"]
 import tensorflow as tf
 import keras
 
-from .decorators import (register_quantize_target, register_no_output_quantizer,
-                         register_aligned_inputs)
+from .layers_base import (register_quantize_target, register_no_output_quantizer,
+                          register_aligned_inputs, apply_buffer_bitwidth, QuantizedLayer)
 from .recorders import TensorRecorder
-from ..tensors import FixedPoint, floor_through, round_log2, MAX_BUFFER_BITWIDTH, pow2
+from ..tensors import FixedPoint, floor_through, round_log2, pow2
 
 
 def shiftmax(logits, axis=-1):
@@ -100,7 +100,7 @@ class Shiftmax(keras.layers.Layer):
 @register_no_output_quantizer
 @register_aligned_inputs
 @tf.keras.utils.register_keras_serializable()
-class QuantizedShiftmax(keras.layers.Layer):
+class QuantizedShiftmax(QuantizedLayer):
     """A quantized layer to do a quantized function similar to the softmax, but
     using base 2 instead of e. So we replace
 
@@ -129,16 +129,14 @@ class QuantizedShiftmax(keras.layers.Layer):
     """
 
     def __init__(self, *args, quant_config=None, **kwargs):
-        super().__init__(*args, **kwargs)
-        self.quant_config = quant_config
-        quant_config = quant_config or dict()
-        self.buffer_bitwidth = MAX_BUFFER_BITWIDTH - 1
-        self.exp_bitwidth = quant_config.get("exp_bitwidth", 10)
+        super().__init__(*args, quant_config=quant_config, **kwargs)
+        self.buffer_bitwidth = apply_buffer_bitwidth(self.quant_config, signed=True)
+        self.exp_bitwidth = self.quant_config.get("exp_bitwidth", 10)
         if self.buffer_bitwidth <= 2 * self.exp_bitwidth:
             raise ValueError(f"exp_bitwidth={self.exp_bitwidth} must be less than "
                              f"half of buffer_size={self.buffer_bitwidth}.")
         # Add objects that will store the shift values.
-        self.input_shift = TensorRecorder()
+        self.input_shift = TensorRecorder(self.name + "/input_shift")
 
     def call(self, x):
         # Raise an error if the inputs are not FixedPoint
@@ -151,7 +149,7 @@ class QuantizedShiftmax(keras.layers.Layer):
         # this because the exponential function will return very big numbers,
         # so fractional ones can be ignored in the ratio with the sum.
         x, shift = x.floor()
-        # update shift values if calibration is enabled
+        # update shift values if recording is enabled
         self.input_shift(shift)
 
         # Since x has been floored, we can directly use its values
@@ -215,8 +213,3 @@ class QuantizedShiftmax(keras.layers.Layer):
         # Since sum_exp > exp, the results are between [0,1].
         # We can therefore rewrite the output as:
         return FixedPoint(outputs.values, self.exp_bitwidth + 1, self.exp_bitwidth)
-
-    def get_config(self):
-        config = super().get_config()
-        config["quant_config"] = self.quant_config
-        return config
