@@ -22,10 +22,11 @@ __all__ = ["ClassToken", "QuantizedClassToken",
 import keras
 import tensorflow as tf
 
-from .decorators import (register_quantize_target, register_no_output_quantizer, rescale_outputs,
-                         tensor_inputs, register_aligned_inputs)
+from .layers_base import (register_quantize_target, register_no_output_quantizer, rescale_outputs,
+                          tensor_inputs, register_aligned_inputs, apply_buffer_bitwidth,
+                          QuantizedLayer)
 from .quantizers import AlignedWeightQuantizer, OutputQuantizer
-from ..tensors import QFloat, FixedPoint, BASE_BUFFER_BITWIDTH
+from ..tensors import QFloat, FixedPoint
 
 
 @tf.keras.utils.register_keras_serializable()
@@ -96,7 +97,7 @@ class AddPositionEmbs(keras.layers.Layer):
 @register_quantize_target(ClassToken)
 @register_no_output_quantizer
 @tf.keras.utils.register_keras_serializable()
-class QuantizedClassToken(ClassToken):
+class QuantizedClassToken(QuantizedLayer, ClassToken):
     """Quantize the :class:`ClassToken` layer, allowing quantization of the output.
 
     Args:
@@ -104,12 +105,10 @@ class QuantizedClassToken(ClassToken):
     """
 
     def __init__(self, *args, quant_config=None, **kwargs):
-        super().__init__(*args, **kwargs)
-        self.quant_config = quant_config
-        quant_config = quant_config or dict()
-        cls_quantizer_cfg = quant_config.get("cls_quantizer", {})
+        super().__init__(*args, quant_config=quant_config, **kwargs)
+        cls_quantizer_cfg = self.quant_config.get("cls_quantizer", {})
         self.cls_quantizer = AlignedWeightQuantizer(name="cls_quantizer", **cls_quantizer_cfg)
-        self.buffer_bitwidth = BASE_BUFFER_BITWIDTH - 1
+        self.buffer_bitwidth = apply_buffer_bitwidth(self.quant_config, signed=True)
 
     @tensor_inputs([QFloat])
     def call(self, inputs):
@@ -123,15 +122,10 @@ class QuantizedClassToken(ClassToken):
 
         return tf.concat([cls_broadcasted, inputs], 1)
 
-    def get_config(self):
-        config = super().get_config()
-        config["quant_config"] = self.quant_config
-        return config
-
 
 @register_quantize_target(AddPositionEmbs)
 @tf.keras.utils.register_keras_serializable()
-class QuantizedAddPositionEmbs(AddPositionEmbs):
+class QuantizedAddPositionEmbs(QuantizedLayer, AddPositionEmbs):
     """Quantize the :class:`AddPositionEmbs` layer, allowing operations in FixedPoint domain.
 
     Args:
@@ -139,18 +133,16 @@ class QuantizedAddPositionEmbs(AddPositionEmbs):
     """
 
     def __init__(self, *args, quant_config=None, **kwargs):
-        super().__init__(*args, **kwargs)
-        self.quant_config = quant_config
-        quant_config = quant_config or dict()
-        out_quant_cfg = quant_config.get("output_quantizer", False)
+        super().__init__(*args, quant_config=quant_config, **kwargs)
+        out_quant_cfg = self.quant_config.get("output_quantizer", False)
         if out_quant_cfg:
             self.out_quantizer = OutputQuantizer(
                 name="output_quantizer", **out_quant_cfg)
         else:
             self.out_quantizer = None
-        pe_quantizer_cfg = quant_config.get("pe_quantizer", {})
+        pe_quantizer_cfg = self.quant_config.get("pe_quantizer", {})
         self.pe_quantizer = AlignedWeightQuantizer(name="pe_quantizer", **pe_quantizer_cfg)
-        self.buffer_bitwidth = BASE_BUFFER_BITWIDTH - 1
+        self.buffer_bitwidth = apply_buffer_bitwidth(self.quant_config, signed=True)
 
     @tensor_inputs([QFloat])
     @rescale_outputs
@@ -161,11 +153,6 @@ class QuantizedAddPositionEmbs(AddPositionEmbs):
         # Add position embeddings
         return inputs + pe
 
-    def get_config(self):
-        config = super().get_config()
-        config["quant_config"] = self.quant_config
-        return config
-
 
 @tf.keras.utils.register_keras_serializable()
 class ExtractToken(keras.layers.Layer):
@@ -173,19 +160,22 @@ class ExtractToken(keras.layers.Layer):
 
     Args:
         token (int): the indice of the token to extract.
+        axis (int, optional): axis over which the user gather the token. Defaults to 1.
 
     """
 
-    def __init__(self, *args, token, **kwargs):
+    def __init__(self, *args, token, axis=1, **kwargs):
         super().__init__(*args, **kwargs)
         self.token = token
+        self.axis = axis
 
     def call(self, inputs):
-        return tf.gather(inputs, self.token, axis=1)
+        return tf.gather(inputs, self.token, axis=self.axis)
 
     def get_config(self):
         config = super().get_config()
         config.update({"token": self.token})
+        config.update({"axis": self.axis})
         return config
 
 
@@ -193,10 +183,13 @@ class ExtractToken(keras.layers.Layer):
 @register_no_output_quantizer
 @register_aligned_inputs
 @tf.keras.utils.register_keras_serializable()
-class QuantizedExtractToken(ExtractToken):
+class QuantizedExtractToken(QuantizedLayer, ExtractToken):
     """ Quantized version of the ExtractToken layer. Accepts only FixedPoint inputs.
     """
 
     @tensor_inputs([FixedPoint])
     def call(self, inputs):
-        return tf.gather(inputs, self.token, axis=1)
+        return tf.gather(inputs, self.token, axis=self.axis)
+
+    def get_config(self):
+        return ExtractToken.get_config(self)
