@@ -1,8 +1,12 @@
 from .align_rescaling import *
 from .invert_batchnorm_pooling import *
 from .fold_batchnorms import *
-from .align_relu import *
+from .remove_reshape import *
 from .remove_zeropadding2d import *
 from .relu_maxpool import *
 from .insert_layer import *
 from .replace_lambda import *
+from .conv_to_dw import *
+from .fold_activations_in_add import *
+from .split_layers_with_relu import *
+from .sanitize import *
