@@ -139,6 +139,8 @@ def find_layers_pairs(model, first_type, second_type):
 
     Args:
         model (keras.Model): a model
+        first_type (keras.Layer): a first layer
+        second_type (keras.Layer): a second layer
 
     Returns:
         dict: layer pairs
@@ -149,9 +151,11 @@ def find_layers_pairs(model, first_type, second_type):
     dst_layers = get_layers_by_type(model, second_type)
 
     # Find 'second_type' layers that have only one inbound layer that is a 'first_type' layer
+    # and  that at the same time it has a single outbound ('second_type').
     for ly in dst_layers:
-        if isinstance(ly.inbound_nodes[0].inbound_layers, first_type):
-            pairs[ly.inbound_nodes[0].inbound_layers] = ly
+        parent_layer = ly.inbound_nodes[0].inbound_layers
+        if isinstance(parent_layer, first_type) and len(parent_layer.outbound_nodes) == 1:
+            pairs[parent_layer] = ly
     return pairs
 
 
@@ -217,3 +221,32 @@ def invert_layer_pairs(model, pairs):
     # Restore model weights
     updated_model.set_weights(model.get_weights())
     return updated_model
+
+
+def get_inbound_layers_config(layer, model_config):
+    """ Retrieve the inbound layers config of 'layer'
+
+    Args:
+        layer (dict): the target layer
+        model_config (dict): the config where to search the inbounds
+
+    Returns:
+        list: the config of the inbound layers
+    """
+    if 'inbound_nodes' in layer:
+        # The config comes from a keras.Model.
+        # First, recover layer names
+        inbound_layer_names = []
+        for inbound_node in inbound_node_generator(layer):
+            if isinstance(inbound_node, dict):
+                inbound_node = inbound_node.values()
+            for connection_info in inbound_node:
+                inbound_layer_names.append(connection_info[0])
+        # Next, match name with layer config
+        inbound_layers = get_layers(model_config, inbound_layer_names)
+    else:
+        # The config comes from a keras.Sequential.
+        # Inbound layer can be found from the index of the layer
+        layer_id = model_config['layers'].index(layer)
+        inbound_layers = [model_config['layers'][layer_id - 1]] if layer_id > 0 else []
+    return inbound_layers
