@@ -18,42 +18,12 @@
 Common utility methods used in quantization models.
 """
 
-__all__ = ['load_model', 'deep_clone_model', 'apply_weights_to_model']
+__all__ = ['apply_weights_to_model', 'get_model_input_dtype']
 
 import warnings
+import tensorflow as tf
 
-from keras.models import clone_model, load_model as kload_model
-
-
-def load_model(model_path, custom_layers=None, compile_model=True):
-    """Loads a model with quantizeml custom layers.
-
-    Args:
-        model_path (str): path of the model to load
-        custom_layers (dict, optional): custom layers to add to the model. Defaults to None.
-        compile_model (bool, optional): whether to compile the model. Defaults to True.
-
-    Returns:
-        keras.Model: the loaded model
-    """
-    return kload_model(model_path, custom_objects=custom_layers, compile=compile_model)
-
-
-def deep_clone_model(model, *args, **kwargs):
-    """Clone a model, assign variable to variable. Useful when a clone function is used,
-    and new layers have not the same number of parameters as the original layer.
-
-    Args:
-        model (keras.Model): model to be cloned
-        args, kwargs (optional): arguments pass to :func:`keras.models.clone_model` function
-
-    Returns:
-        keras.Model: the cloned model
-    """
-    new_model = clone_model(model, *args, **kwargs)
-    variables_dict = {var.name: var for var in model.variables}
-    apply_weights_to_model(new_model, variables_dict, False)
-    return new_model
+from ..layers import StatefulRecurrent
 
 
 def apply_weights_to_model(model, weights, verbose=True):
@@ -83,3 +53,20 @@ def apply_weights_to_model(model, weights, verbose=True):
                 break
         if not value_applied and verbose:
             warnings.warn(f"Variable '{key}' not found in the model.")
+
+
+def get_model_input_dtype(model):
+    """Retrieve the common input dtype for a model
+
+    Handle image like samples (channels in [1, 3]) as uint8, recurrent TENNs as int16 and
+    other as int8
+
+    Args:
+        model (keras.Model or keras.Sequential): the model to get the input dtype.
+
+    Returns:
+        tf.dtype: the expected input type for the model
+    """
+    if any(isinstance(ly, StatefulRecurrent) for ly in model.layers):
+        return tf.int16
+    return tf.uint8 if model.input_shape[-1] in [1, 3] else tf.int8
