@@ -16,12 +16,14 @@
 # ******************************************************************************
 import tensorflow as tf
 
-from .qtensor import QTensor, saturate, round_through, floor_through, pow2, ceil_log2
+from .qtensor import (
+    QTensor, saturate, round_through, floor_through, pow2,
+    ceil_log2, slice_tensor_by_index)
 from ..debugging import assert_equal, assert_none_equal, assert_less, assert_less_equal
 
 
 class FixedPoint(QTensor):
-    """A Tensor of integer values representing fixed-point numbers
+    r"""A Tensor of integer values representing fixed-point numbers
 
     The value_bits parameter sets the maximum integer values that can be stored:
 
@@ -39,12 +41,6 @@ class FixedPoint(QTensor):
     frac_bits: tf.Tensor = 0.
 
     def __init__(self, values, value_bits, frac_bits):
-        # We store fractional bits in a float tensor to speed up calculations
-        if isinstance(frac_bits, tf.Tensor):
-            self.frac_bits = tf.cast(frac_bits, tf.float32)
-        else:
-            self.frac_bits = tf.convert_to_tensor(frac_bits, tf.float32)
-        self.value_bits = value_bits
         # We store integer values in a float tensor to speed up calculations
         if isinstance(values, tf.Tensor):
             values = tf.cast(values, tf.float32)
@@ -52,6 +48,13 @@ class FixedPoint(QTensor):
             values = tf.convert_to_tensor(values, dtype=tf.float32)
         # Clamp to fixed-point boundaries
         self.values = saturate(values, tf.cast(value_bits, tf.float32))
+        # We store fractional bits in a float tensor to speed up calculations
+        if isinstance(frac_bits, tf.Tensor):
+            self.frac_bits = tf.cast(frac_bits, tf.float32)
+        else:
+            self.frac_bits = tf.convert_to_tensor(frac_bits, tf.float32)
+        self.value_bits = value_bits
+
         self.shape = self.values.shape
 
     @property
@@ -73,7 +76,7 @@ class FixedPoint(QTensor):
         From that it deduces the rightmost fractional bit indices.
 
         The resulting frac_bits can be a negative number if the ranges are higher than the biggest
-        integer than can be represented with the specified value bits).
+        integer that can be represented with the specified value bits.
 
         If specified, the maximum fractional bits are clamped to the available value_bits.
 
@@ -136,10 +139,10 @@ class FixedPoint(QTensor):
                 # The input does not need to be quantized
                 frac_bits = 0
             else:
-                frac_bits = FixedPoint.max_frac_bits(value_bits, tf.abs(x))
+                frac_bits = FixedPoint.max_frac_bits(value_bits, tf.math.reduce_max(tf.abs(x)))
         # Project float into fixed-point representation space
         x_scaled = tf.math.multiply(x, pow2(frac_bits), name="quantize")
-        # Round or ceil to obtain integer values
+        # Round to obtain integer values
         values = round_through(x_scaled)
         return FixedPoint(values, value_bits, frac_bits)
 
@@ -269,21 +272,11 @@ class FixedPoint(QTensor):
         Returns:
             :obj:`FixedPoint`: the upscaled FixedPoint
         """
-        if value_bits is None:
-            value_bits = self.value_bits
         frac_bits = tf.cast(frac_bits, tf.float32)
         assert_less_equal(self.frac_bits, frac_bits,
                           f"Cannot increase {self.name} precision because the target precision "
                           f"({frac_bits}) is lower than the current one ({self.frac_bits})")
-        # Evaluate the shift to apply to reach the target precision
-        shift = frac_bits - self.frac_bits
-        assert_less(shift, value_bits,
-                    f"Cannot upscale {self.name} to {frac_bits} fractional bits as it saturates its"
-                    f" {value_bits}-bit buffer.")
-        # Apply shift
-        values = tf.math.multiply(self.values, pow2(shift), name="upscale")
-        # return a new FixedPoint with the target precision and bitwidth
-        return FixedPoint(values, value_bits, frac_bits), shift
+        return self.rescale(frac_bits, value_bits)
 
     def expand(self, value_bits):
         """Expand the FixedPoint to the specified bitwidth
@@ -449,6 +442,36 @@ class FixedPoint(QTensor):
         raise TypeError(
             f"Unsupported operand type(s) for /: 'FixedPoint' and '{type(other)}'")
 
+    def __getitem__(self, idx):
+
+        r"""Retrieve a slice or element from the FixedPoint tensor.
+
+        This function allows for slicing or indexing the FixedPoint tensor in a similar way
+        to a standard TensorFlow tensor. It handles the slicing of both the values
+        and the frac_bits of the FixedPoint tensor, ensuring that the tensor
+        representation remains consistent.
+
+        Args:
+            idx (int, slice, or tuple): The index or slice to retrieve. It can be an integer,
+                a slice, or a tuple of integers/slices. The tuple can
+                also include the Ellipsis (`...`) to denote a slice
+                over remaining dimensions.
+
+        Returns:
+            FixedPoint: A new FixedPoint instance containing the sliced values and corresponding
+                frac_bits.
+        """
+
+        sliced_values = self.values[idx]
+
+        if self.per_tensor:
+            return FixedPoint(sliced_values, self.value_bits, self.frac_bits)
+
+        else:
+            values_rank = len(self.shape)
+            sliced_frac_bits = slice_tensor_by_index(idx, values_rank, self.frac_bits)
+            return FixedPoint(sliced_values, self.value_bits, sliced_frac_bits)
+
     def __pow__(self, power):
         if isinstance(power, int):
             if power == 0:
