@@ -16,7 +16,7 @@
 # ******************************************************************************
 import tensorflow as tf
 
-from .qtensor import QTensor
+from .qtensor import QTensor, slice_tensor_by_index
 from .fixed_point import FixedPoint
 from ..debugging import assert_equal
 
@@ -90,7 +90,7 @@ class QFloat(QTensor):
 
     @staticmethod
     def optimal_scales(ranges, value_bits):
-        """Evaluates the optimal QFloat scales for quantization ranges.
+        r"""Evaluates the optimal QFloat scales for quantization ranges.
 
         We choose the optimal quantization range for a given bitwidth as:
 
@@ -193,6 +193,32 @@ class QFloat(QTensor):
         # Return a new QFloat with updated values and the same scale
         return QFloat(fp, self.scales), shift
 
+    def expand(self, value_bits):
+        """Expand the QFloat to the specified bitwidth
+
+        This returns an equivalent QFloat with a higher or equal number of
+        value bits and a scalar fractional bit corresponding to the maximum of
+        the initial fractional bits on all channels. The scales remains unchanged.
+
+        This is mostly used to recover a per-tensor QFloat that has been
+        compressed to a lower number of value bits.
+
+        Note that even if the frac_bits are aligned the scales remained unchanged.
+
+        Args:
+            value_bits (int): the target value_bits
+
+        Returns:
+            tuple(:obj:`QFloat`, tf.Tensor): a new QFloat with expanded
+            fractional bits and the shift that was applied.
+        """
+        if value_bits < self.value_bits:
+            raise ValueError(
+                f"Cannot reduce {self.name} bitwidth from {self.value_bits} to {value_bits}:"
+                " use a quantizer instead.")
+        max_frac_bits = tf.reduce_max(self.frac_bits)
+        return self.upscale(max_frac_bits, value_bits)
+
     def promote(self, bits):
         """Increase the number of value bits
 
@@ -215,8 +241,48 @@ class QFloat(QTensor):
         raise TypeError(
             f"Unsupported operand type(s) for +: 'QFloat' and '{type(other)}'")
 
+    def __sub__(self, other):
+        if isinstance(other, QFloat):
+            # Check that self and other have the same scales
+            assert_equal(self.scales, other.scales,
+                         message=f"{self.name} and {other.name} have different scales.")
+            # Return a new QFloat
+            return QFloat(self.fp - other.fp, self.scales)
+        raise TypeError(
+            f"Unsupported operand type(s) for -: 'QFloat' and '{type(other)}'")
+
     def __truediv__(self, other):
         # We only support the division of the scales by a compatible type
         div_scales = self.scales / other
         # Return a Qfloat with updated scales
         return QFloat(self.fp, div_scales)
+
+    def __getitem__(self, idx):
+
+        r"""Retrieve a slice or element from the QFloat tensor.
+
+        This function allows for slicing or indexing the QFloat tensor in a similar way
+        to a standard TensorFlow tensor. It handles the slicing of both the fixedpoint
+        and the scales of the QFloat tensor, ensuring that the tensor
+        representation remains consistent.
+
+        Args:
+            idx (int, slice, or tuple): The index or slice to retrieve. It can be an integer,
+                a slice, or a tuple of integers/slices. The tuple can
+                also include the Ellipsis (`...`) to denote a slice
+                over remaining dimensions.
+
+        Returns:
+            QFloat: A new QFloat instance containing the sliced values and corresponding
+                scales.
+        """
+
+        sliced_fp = self.fp[idx]
+
+        if self.per_tensor:
+            return QFloat(sliced_fp, self.scales)
+
+        else:
+            values_rank = len(self.shape)
+            sliced_scales = slice_tensor_by_index(idx, values_rank, self.scales)
+            return QFloat(sliced_fp, sliced_scales)
