@@ -109,6 +109,14 @@ class QTensor(tf.experimental.ExtensionType):
         """
         raise NotImplementedError
 
+    def __getitem__(self):
+        """Retrieve a slice or element from the QTensor
+
+        Returns:
+            :obj:`QTensor`: the sliced QTensor
+        """
+        raise NotImplementedError
+
     def clone(self):
         """Returns a copy of the QTensor
 
@@ -208,3 +216,70 @@ def round_log2(x):
             representing the closest PoT exponents.
     """
     return round_through(tf.experimental.numpy.log2(tf.cast(x, tf.float32)))
+
+
+def convert_ellipsis_into_slices(values_rank, idx):
+    """
+    Converts an Ellipsis (`...`) if found in the indexing tuple into a series of slice objects
+    that cover the remaining dimensions of the tensor.
+
+    Args:
+        values_rank (int): The rank (number of dimensions) of the tensor being indexed.
+        idx (tuple): The indexing tuple that may contain an Ellipsis.
+
+    Returns:
+        tuple: A new indexing tuple with the Ellipsis replaced by appropriate slices.
+    """
+    new_idx = ()
+    for element in idx:
+        if element == ...:
+            for _ in range(values_rank - len(idx) + 1):
+                new_idx += (slice(None, None, None),)
+        else:
+            new_idx += (element,)
+    return new_idx
+
+
+def slice_tensor_by_index(idx, values_rank, tensor):
+    """
+    Slices the frac_bits or scales tensor when slicing a QTensor.
+
+    Args:
+        idx (tuple/slice/int/Ellipsis): The indexing tuple.
+        values_rank (int): The rank (number of dimensions) of the values tensor.
+        tensor (tf.Tensor): The frac_bits/scale tensor to slice.
+
+    Returns:
+        tf.Tensor: the sliced frac_bits/scale tensor tensor.
+    """
+
+    tensor_rank = tf.rank(tensor)
+    if isinstance(idx, tuple):
+        # Convert Ellipsis into actual slices
+        if any(x is Ellipsis for x in idx):
+            idx = convert_ellipsis_into_slices(values_rank, idx)
+
+        # Case where the number of indices matches the rank of the values tensor
+        if values_rank == len(idx):
+            new_fb_tuple = idx[-tensor_rank:]
+            return tensor[new_fb_tuple]
+
+        # Case where the frac_bits/scales tensor has fewer dimensions than the remaining positions
+        elif values_rank - len(idx) - tensor_rank >= 0:
+            return tensor
+
+        # Case where the frac_bits/scales tensor needs more dimensions
+        else:
+            m = tensor_rank - (values_rank - len(idx))
+            fb_ind = idx[-m:]
+            fb_ind += (Ellipsis,)
+            return tensor[fb_ind]
+
+    # Case where the index is not a tuple (int, ellipsis, scalar)
+    # and values values has more dimensions than the frac_bits/scales tensor
+    elif values_rank > tensor_rank:
+        return tensor
+
+    # Case where the values and tensor have the same dimensions
+    else:
+        return tensor[idx]
