@@ -215,7 +215,8 @@ def fp_fp_multiply(x: FixedPoint, y: FixedPoint, name=None):
     Returns:
         :obj:`FixedPoint`: an object with the same shape as x
     """
-    return x * y
+    values = tf.math.multiply(x.values, y.values, name)
+    return FixedPoint(values, x.value_bits, x.frac_bits + y.frac_bits)
 
 
 @tf.experimental.dispatch_for_api(tf.math.multiply)
