@@ -366,98 +366,96 @@ def qf_qf_convolution(
     return QFloat(conv_fp, conv_scales)
 
 
-@tf.experimental.dispatch_for_api(tf.nn.conv2d_transpose)
+@tf.experimental.dispatch_for_api(keras.backend.conv2d_transpose)
 def fp_fp_conv2d_transpose(
-        input: FixedPoint,
-        filters: FixedPoint,
-        output_shape, strides=None, padding="VALID", data_format=None, dilations=None, name=None):
+        x: FixedPoint,
+        kernel: FixedPoint,
+        output_shape, strides=(1, 1), padding='valid', data_format=None, dilation_rate=(1, 1)):
     """The transpose of conv2d.
 
     This operation is sometimes called "deconvolution" after (Zeiler et al., 2010), but is
     really the transpose (gradient) of atrous_conv2d rather than an actual deconvolution.
 
     Args:
-        input (:obj:`FixedPoint`): A 4-D FixedPoint of shape [batch, height, width, in_channels] for
+        x (:obj:`FixedPoint`): A 4-D FixedPoint of shape [batch, height, width, in_channels] for
             NHWC data format or [batch, in_channels, height, width] for NCHW data format.
-        filters (:obj:`FixedPoint`): A FixedPoint with a [height, width, out_channels, in_channels]
-            shape. Filter's in_channels dimension must match that of input.
+        kernel (:obj:`FixedPoint`): A FixedPoint with a [height, width, out_channels, in_channels]
+            shape. Kernel's in_channels dimension must match that of input.
         output_shape (tf.Tensor): A 1-D Tensor representing the output shape of the
             deconvolution op.
-        strides (list, optional): Sequence of N ints >= 1. Specifies the output stride.
-            Defaults to None.
-        padding (str, optional): A string, either `"VALID"` or `"SAME"`. The padding algorithm.
-            Defaults to "VALID"
+        strides (list, optional): Sequence of 2 ints >= 1. Specifies the output stride.
+            Defaults to (1, 1).
+        padding (str, optional): A string, either `'valid'` or `'same'`. The padding algorithm.
+            Defaults to 'valid'
         data_format (str, optional): Specifies whether the channel dimension of
             the `input` and output is the last dimension (default, or if `data_format`
             does not start with "NC"), or the second dimension (if `data_format`
             starts with "NC"). For N=1, the valid values are "NWC" (default) and
             "NCW". For N=2, the valid values are "NHWC" (default) and "NCHW".
             For N=3, the valid values are "NDHWC" (default) and "NCDHW".
-        dilations (list, optional): Alias for dilation_rate. Sequence of N ints >= 1.
-            Specifies the filter upsampling/input downsampling rate. Defaults to None.
-        name (str, optional): the name for the Tensorflow ops. Defaults to None.
+        dilation_rate (list, optional): Sequence of 2 ints >= 1.
+            Specifies the filter upsampling/input downsampling rate. Defaults to (1, 1).
 
     Returns:
         :obj:`FixedPoint`: a FixedPoint containing the output values.
     """
     # Input must be quantized per-tensor because the products are eventually summed
-    input.assert_per_tensor()
+    x.assert_per_tensor()
 
     # Transpose filter values so that they have the shape expected by the tensorflow op:
     #  (h, w, filters, channels)
-    filter_values = tf.transpose(filters.values, (0, 1, 3, 2))
+    filter_values = tf.transpose(kernel.values, (0, 1, 3, 2))
 
     # Do conv2d_transpose on values
-    conv_values = tf.nn.conv2d_transpose(input.values, filter_values, output_shape, strides,
-                                         padding, data_format, dilations, name)
+    conv_values = keras.backend.conv2d_transpose(x.values, filter_values, output_shape, strides,
+                                                 padding, data_format, dilation_rate)
 
     # Return a new FixedPoint
-    return FixedPoint(conv_values, input.value_bits, input.frac_bits + filters.frac_bits)
+    return FixedPoint(conv_values, x.value_bits, x.frac_bits + kernel.frac_bits)
 
 
-@tf.experimental.dispatch_for_api(tf.nn.conv2d_transpose)
+@tf.experimental.dispatch_for_api(keras.backend.conv2d_transpose)
 def fp_qf_conv2d_transpose(
-        input: FixedPoint,
-        filters: QFloat,
-        output_shape, strides=None, padding="VALID", data_format=None, dilations=None, name=None):
+        x: FixedPoint,
+        kernel: QFloat,
+        output_shape, strides=(1, 1), padding='valid', data_format=None, dilation_rate=(1, 1)):
     """The transpose of conv2d.
 
     This operation is sometimes called "deconvolution" after (Zeiler et al., 2010), but is
     really the transpose (gradient) of atrous_conv2d rather than an actual deconvolution.
 
     Args:
-        input (:obj:`FixedPoint`): A 4-D FixedPoint of shape [batch, height, width, in_channels] for
+        x (:obj:`FixedPoint`): A 4-D FixedPoint of shape [batch, height, width, in_channels] for
             NHWC data format or [batch, in_channels, height, width] for NCHW data format.
-        filters (:obj:`QFloat`): A QFloat of shape [height, width, out_channels, in_channels]
-            shape. Filter's in_channels dimension must match that of input.
+        kernel (:obj:`QFloat`): A QFloat of shape [height, width, out_channels, in_channels]
+            shape. Kernel's in_channels dimension must match that of input.
         output_shape (tf.Tensor): A 1-D Tensor representing the output shape of the
             deconvolution op.
-        strides (list, optional): Sequence of N ints >= 1. Specifies the output stride.
-            Defaults to None.
-        padding (str, optional): A string, either `"VALID"` or `"SAME"`. The padding algorithm.
-            Defaults to "VALID"
+        strides (list, optional): Sequence of 2 ints >= 1. Specifies the output stride.
+            Defaults to (1, 1).
+        padding (str, optional): A string, either `'valid'` or `'same'`. The padding algorithm.
+            Defaults to 'valid'
         data_format (str, optional): Specifies whether the channel dimension of
             the `input` and output is the last dimension (default, or if `data_format`
             does not start with "NC"), or the second dimension (if `data_format`
             starts with "NC"). For N=1, the valid values are "NWC" (default) and
             "NCW". For N=2, the valid values are "NHWC" (default) and "NCHW".
             For N=3, the valid values are "NDHWC" (default) and "NCDHW".
-        dilations (list, optional): Alias for dilation_rate. Sequence of N ints >= 1.
-            Specifies the filter upsampling/input downsampling rate. Defaults to None.
-        name (str, optional): the name for the Tensorflow ops. Defaults to None.
+        dilation_rate (list, optional): Sequence of 2 ints >= 1.
+            Specifies the filter upsampling/input downsampling rate. Defaults to (1, 1).
 
     Returns:
         :obj:`QFloat`: a QFloat containing the output values.
     """
     # Input must be quantized per-tensor because the products are eventually summed
-    input.assert_per_tensor()
+    x.assert_per_tensor()
 
     # Do conv2d_transpose on inputs and filter inner FixedPoint
-    conv_values = tf.nn.conv2d_transpose(input, filters.fp, output_shape, strides,
-                                         padding, data_format, dilations, name)
+    conv_values = keras.backend.conv2d_transpose(x, kernel.fp, output_shape, strides,
+                                                 padding, data_format, dilation_rate)
 
     # Return a new QFloat
-    return QFloat(conv_values, filters.scales)
+    return QFloat(conv_values, kernel.scales)
 
 
 @tf.experimental.dispatch_for_api(tf.nn.max_pool)
