@@ -17,7 +17,7 @@
 import tensorflow as tf
 from typing import List
 
-from ...debugging import assert_equal, assert_less_equal
+from ...debugging import assert_equal, assert_less_equal, assert_enabled
 from .. import FixedPoint, QFloat
 
 
@@ -229,6 +229,63 @@ def qf_concat(values: List[QFloat], axis, name="concat"):
     return QFloat(output_fp, output_scales)
 
 
+@tf.experimental.dispatch_for_api(tf.stack)
+def fp_stack(values: List[FixedPoint], axis, name="stack"):
+    """Stack FixedPoint tensors along one dimension.
+
+    Args:
+        values (List of :obj:`FixedPoint`): List of FixedPoint tensors
+            to stack.
+        axis (list): Dimension along which to stack.
+        name (str, optional): the name for the Tensorflow ops.
+            Defaults to "stack".
+
+    Returns:
+        :obj:`FixedPoint`: the stacked output FixedPoint.
+    """
+    # For now we only support stack of one or two elements
+    assert_equal(
+        len(values),
+        2, f"We only support stack of two FixedPoint. \
+           Receives {len(values)} tensors as input.")
+
+    # Promote to the higher value bits
+    if values[0].value_bits < values[1].value_bits:
+        values[0] = values[0].promote(values[1].value_bits)
+    elif values[0].value_bits > values[1].value_bits:
+        values[1] = values[1].promote(values[0].value_bits)
+
+    # infer unsigned axis
+    offset = values[0].shape.ndims + 1
+    if axis < 0:
+        dim = axis + offset
+    else:
+        dim = axis
+
+    last_dim = dim == offset - 1
+
+    # First stack the FixedPoint.values
+    values_out = tf.stack([values[0].values, values[1].values], axis, name)
+    # When dim is last dimension, the two FixedPoint should be per-tensor otherwise their
+    # frac_bits should be equal.
+    if last_dim:
+        if assert_enabled():
+            assert (values[0].per_tensor and values[1].per_tensor), (f"The two \
+                    FixedPoint must be per-tensor when stacking on the last dim.\
+                    Receives: a.per-tensor:{values[0].per_tensor} and \
+                    b.per-tensor:{values[1].per_tensor}")
+        frac_bits_a = values[0].frac_bits
+        frac_bits_b = values[1].frac_bits
+        # Concatenate frac_bits
+        frac_bits_out = tf.stack([frac_bits_a, frac_bits_b], 0, f"{name}_frac_bits")
+    else:
+        assert_equal(values[0].frac_bits, values[1].frac_bits, message=f"The two\
+                FixedPoint must have the same frac_bits. Receives {values[0].frac_bits}\
+                and {values[1].frac_bits}")
+        frac_bits_out = values[0].frac_bits
+    return FixedPoint(values_out, values[0].value_bits, frac_bits_out)
+
+
 @tf.experimental.dispatch_for_api(tf.expand_dims)
 def fp_expand_dims(input: FixedPoint, axis, name=None):
     """Returns a tensor with a length 1 axis inserted at index `axis`.
@@ -243,11 +300,18 @@ def fp_expand_dims(input: FixedPoint, axis, name=None):
         FixedPoint: a tensor with the same data as `input`, with an additional dimension inserted at
             the index specified by `axis`.
     """
-    # Only support per-tensor inputs
-    input.assert_per_tensor()
+    # Evaluate the index of the dimension on which we expand
+    rank_input = input.shape.ndims
+    if axis < 0:
+        expanded_dim = axis + rank_input + 1
+    else:
+        expanded_dim = axis
+    # Only support per-tensor inputs if the expanded along the last dim
+    if expanded_dim == rank_input:
+        input.assert_per_tensor()
 
     # Expand dimension on values
-    values = tf.expand_dims(input.values, axis, name)
+    values = tf.expand_dims(input.values, expanded_dim, name)
 
     # Return a new FixedPoint
     return FixedPoint(values, input.value_bits, input.frac_bits)
@@ -267,8 +331,15 @@ def qf_expand_dims(input: QFloat, axis, name=None):
         QFloat: a tensor with the same data as `input`, with an additional dimension inserted at
             the index specified by `axis`.
     """
-    # Only support per-tensor inputs
-    input.assert_per_tensor()
+    # Evaluate the index of the dimension on which we expand
+    rank_input = input.shape.ndims
+    if axis < 0:
+        expanded_dim = axis + rank_input + 1
+    else:
+        expanded_dim = axis
+    # Only support per-tensor inputs if the expanded along the last dim
+    if expanded_dim == rank_input:
+        input.assert_per_tensor()
 
     # Expand dimension on inner FixedPoint
     expanded_fp = tf.expand_dims(input.fp, axis, name)
@@ -283,7 +354,7 @@ def fp_gather(params: FixedPoint, indices, validate_indices=None,
     """Gather slices from params along axis.
 
     Args:
-        params (:obj:`FixedPoint`, :obj:`QFloat`): the input QTensor.
+        params (:obj:`FixedPoint`): the input FixedPoint.
         indices (int, list): the indices to gather.
         validate_indices (bool, optional): whether to validate the indices. Defaults to None.
         name (str, optional): the name for the Tensorflow ops. Defaults to None.
@@ -291,14 +362,43 @@ def fp_gather(params: FixedPoint, indices, validate_indices=None,
         batch_dims (int, optional): the number of batch dimensions to keep. Defaults to 0.
 
     Returns:
-        :obj:`QTensor`: a QTensor containing the gathered values.
+        :obj:`FixedPoint`: a FixedPoint containing the gathered values.
     """
-    # We do not support gather along the last axis if the QTensor is per-axis
-    if axis in (-1, len(params.shape)):
-        params.assert_per_tensor()
+    # If gather along the last axis and params is per axis, select only the targeted frac bits.
+    if not params.per_tensor and axis in (-1, len(params.shape) - 1):
+        frac_bits = params.frac_bits[indices]
+    else:
+        frac_bits = params.frac_bits
     # Create a new QTensor, gather the indices in a desired axis
     x_gather = tf.gather(params.values, indices, validate_indices, axis, batch_dims, name)
-    return FixedPoint(x_gather, params.value_bits, params.frac_bits)
+    return FixedPoint(x_gather, params.value_bits, frac_bits)
+
+
+@tf.experimental.dispatch_for_api(tf.compat.v1.gather)
+def qf_gather(params: QFloat, indices, validate_indices=None, name=None, axis=None, batch_dims=0):
+    """Gather slices from params along axis.
+
+    Args:
+        params (:obj:`QFloat`): the input QFloat.
+        indices (int, list): the indices to gather.
+        validate_indices (bool, optional): whether to validate the indices. Defaults to None.
+        name (str, optional): the name for the Tensorflow ops. Defaults to None.
+        axis (int, optional): the axis to gather along. Defaults to None.
+        batch_dims (int, optional): the number of batch dimensions to keep. Defaults to 0.
+
+    Returns:
+        :obj:`QFloat`: a QFloat containing the gathered values.
+    """
+    # If gather along the last axis and params is per axis, select only the targeted frac bits
+    # and scales.
+    if not params.per_tensor and axis in (-1, len(params.shape) - 1):
+        scales = params.scales[indices]
+    else:
+        scales = params.scales
+    # Create a new QTensor, gather the indices in a desired axis
+
+    xfp_gather = tf.gather(params.fp, indices, validate_indices, axis, batch_dims, name)
+    return QFloat(xfp_gather, scales)
 
 
 @tf.experimental.dispatch_for_api(tf.pad)
@@ -357,3 +457,98 @@ def qf_pad(tensor: QFloat,
                  "Padding value is not aligned with the tensor.")
     padded_fp = tf.pad(tensor.fp, paddings, mode, constant_values.fp, name)
     return QFloat(padded_fp, tensor.scales)
+
+
+@tf.experimental.dispatch_for_api(tf.tile)
+def fp_tile(input: FixedPoint, multiples, name=None):
+    """Constructs a tensor by tiling a given tensor.
+
+    Args:
+        input (FixedPoint): a `Tensor`.
+        multiples (tf.Tensor): Must be one of the following types: `int32`, `int64`.
+            1-D. Length must be the same as the number of dimensions in `input`
+        name (str, optional): name of the output `Tensor`. Defaults to None.
+
+    Returns:
+        FixedPoint: a tensor with the same data as `input`.
+    """
+    # If the duplication should occurs on the last axis the input should be per tensor
+    if multiples[-1] != 1:
+        input.assert_per_tensor()
+
+    # tile input.values "multiples" time
+    values = tf.tile(input.values, multiples, name)
+
+    # Return a new FixedPoint
+    return FixedPoint(values, input.value_bits, input.frac_bits)
+
+
+@tf.experimental.dispatch_for_api(tf.tile)
+def qf_tile(input: QFloat, multiples, name=None):
+    """Constructs a tensor by tiling a given tensor.
+
+    Args:
+        input (QFloat): a `Tensor`.
+        multiples (tf.Tensor): Must be one of the following types: `int32`, `int64`.
+            1-D. Length must be the same as the number of dimensions in `input`
+        name (str, optional): name of the output `Tensor`. Defaults to None.
+
+
+    Returns:
+        QFloat: a tensor with the same data as `input`.
+    """
+
+    # If the duplication should occurs on the last axis the input should be per tensor
+    if multiples[-1] != 1:
+        input.assert_per_tensor()
+
+    # tile input.fp "multiples" time
+    values = tf.tile(input.fp, multiples, name)
+
+    # Return a new QFloat
+    return QFloat(values, input.scales)
+
+
+@tf.experimental.dispatch_for_api(tf.zeros_like)
+def fp_zeros_like(input: FixedPoint, dtype=None, name=None):
+    """Creates a FixedPoint with all elements set to zero and the same frac_bits and shape as
+    input.
+
+    Args:
+        input (FixedPoint): a `FixedPoint`.
+        dtype (tf.dtype): A type for the returned `Tensor`. Must be `float16`, `float32`,
+            `float64`, `int8`, `uint8`, `int16`, `uint16`, `int32`, `int64`, `complex64`,
+            `complex128`, `bool` or `string` (optional).
+        name (str, optional): name of the output `Tensor`. Defaults to None.
+
+    Returns:
+        FixedPoint: a tensor with the same data as `input`.
+    """
+    # Create a tf.Tensor with all element set to zeros
+    values = tf.zeros_like(input.values, dtype, name)
+
+    # Return a new FixedPoint
+    return FixedPoint(values, input.value_bits, input.frac_bits)
+
+
+@tf.experimental.dispatch_for_api(tf.zeros_like)
+def qf_zeros_like(input: QFloat, dtype=None, name=None):
+    """Creates a QFloat with all elements set to zero and the same scales and shape as
+    input.
+
+    Args:
+        input (QFloat): a `QFloat`.
+        dtype (tf.dtype): A type for the returned `Tensor`. Must be `float16`, `float32`,
+            `float64`, `int8`, `uint8`, `int16`, `uint16`, `int32`, `int64`, `complex64`,
+            `complex128`, `bool` or `string` (optional).
+        name (str, optional): name of the output `Tensor`. Defaults to None.
+
+
+    Returns:
+        QFloat: a tensor with the same data as `input`.
+    """
+    # Create a tf.Tensor with all element set to zeros
+    values = tf.zeros_like(input.fp, dtype, name)
+
+    # Return a new QFloat
+    return QFloat(values, input.scales)
